#!/bin/bash

set -e
set -o pipefail

mkdir -p /var/log/installer

# 拷贝配置文件
do_copy_config() {

  # sp1 使用 automatic
  if grep -q automatic /proc/cmdline; then
    if grep -q "url=" /proc/cmdline; then
        if [[ -f /cdrom/ky-installer.cfg ]]; then
                cp /cdrom/ky-installer.cfg /tmp
        fi
        if grep -q "cfg=" /proc/cmdline; then
                cmd_cfg=$(cat /proc/cmdline)
                url_ip=$(echo ${cmd_cfg##*url=}| awk '{print $1}')
                url_ip=${url_ip%/*}
                cfg=$(echo ${cmd_cfg##*cfg=}| awk '{print $1}')
                wget ${url_ip}/${cfg} -O /tmp/ky-installer.cfg
        fi
    else
        if grep -q "cfg=" /proc/cmdline; then
                cmd_cfg=$(cat /proc/cmdline)
                cfg=$(echo ${cmd_cfg##*cfg=}| awk '{print $1}')
                cp /cdrom/${cfg}  /tmp/ky-installer.cfg
        else
                if [[ -f /cdrom/ky-installer.cfg ]]; then
                        cp /cdrom/ky-installer.cfg /tmp
		elif [[ -f /cdrom/kyple-installer.cfg ]]; then
			cp /cdrom/kyple-installer.cfg /tmp/ky-installer.cfg
                else
                        echo "ky-installer.cfg does not exist"
                        echo "您现在用的是自动安装流程，请将ky-installer.cfg放到iso根目录下"
                        exit 1
                fi

        fi
    fi
  # 990/9A0 使用此设置，ple-mode 审核模式，test-mode 全自动安装模式
  elif grep -qE 'ple-mode|test-mode' /proc/cmdline; then
    [[ -f /cdrom/kyple-installer.cfg ]] && cp /cdrom/kyple-installer.cfg /tmp/ky-installer.cfg
  else
    cp /usr/share/kylin-os-installer/ky-installer.cfg /tmp
  fi
  echo "ky-installer.cfg exist"
  if [[ -f /tmp/ky-installer.cfg ]]; then
	cp /tmp/ky-installer.cfg /var/log/installer/ky-installer.cfg || echo "拷贝/tmp/ky-installer.cfg失败"
  fi
}

do_copy_config >/var/log/installer/kylin-os-installer-detail.log

# 使用绝对路径，方便在 /target 中加载
cp utils /tmp
. /tmp/utils

case ${LANGUAGES} in
        'zh_CN')
                echo "系统安装前准备..." >${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Preparation before system installation..." >${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "མ་ལག་སྒྲིག་སྦྱོར་གོང་གི་གྲ་སྒྲིག།..." >${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "系統安裝前準備..." >${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠢᠰᠲ᠋ᠡᠮ ᠲᠣᠨᠣᠭᠯᠠᠬᠤ ᠠᠴᠠ ᠡᠮᠦᠨᠡᠬᠢ ᠪᠡᠯᠡᠳᠬᠡᠯ..." >${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "系统安装前准备..." >${LOG_PROGRESS_FILE}
                ;;
esac

if grep -q automatic /proc/cmdline; then
        data_unformat=$(get_value data-unformat)
        if [[ "${data_unformat}" == "true" ]]; then
                save_disk=$(get_value devpath)
                if ! grep -q "save-data=${save_disk}" /var/log/installer/os-prober.log; then
                        echo "您选择了保留用户数据安装，但机器系统不满足保留用户数据要求，不能进行保留用户数据安装" >>/var/log/installer/kylin-os-installer.log
                        exit 1
                fi
        fi
fi

# 运行 prepare 脚本
do_prepare() {
  mkdir -p /var/log/installer

  if [[ -d /cdrom/hooks/prepare ]]; then
    case ${LANGUAGES} in
        'zh_CN')
                echo "执行安装前prepare定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Customize prepare scripts before installation execution..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "ལག་བསྟར་སྒྲིག་སྦྱོར་གོང་prepareངེས་སྲོལ་འཁྲབ་དེབ། ..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "執行安裝前prepare定制腳本..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠬᠤ ᠠᠴᠠ ᠡᠮᠦᠨᠡᠬᠢ prepare ᠦᠢᠯᠡᠳᠬᠦ ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "执行安装前prepare定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
    esac
    run_scripts_hooks /cdrom/hooks/prepare || do_error hooks-prepare
    case ${LANGUAGES} in
        'zh_CN')
                echo "安装前prepare定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Prepare custom script execution before installation completed" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "སྒྲིག་སྦྱོར་གོང་prepareངེས་སྲོལ་འཁྲབ་དེབ་ལག་བསྟར་ལེགས་སྒྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "安裝前prepare定制腳本執行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠬᠤ ᠠᠴᠠ ᠡᠮᠦᠨᠡ prepare ᠦᠢᠯᠡᠳᠬᠦ ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠢ᠋ ᠬᠡᠷᠡᠭᠵᠢᠭᠦᠯᠵᠦ ᠳᠠᠭᠤᠰᠭᠠᠪᠠ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "安装前prepare定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
    esac
  fi
  hooks=$(get_value hooks)
  if [[ -n $hooks ]]; then
#  if grep -q "hooks=" /proc/cmdline; then
    if grep -q "url=" /proc/cmdline; then
            cmd_cfg=$(cat /proc/cmdline)
            url_ip=$(echo ${cmd_cfg##*url=}| awk '{print $1}')
            url_ip=${url_ip%/*}
#            hooks=$(echo ${cmd_cfg##*hooks=}| awk '{print $1}')
            wget -P/tmp ${url_ip}/${hooks}
            tar -xf /tmp/${hooks} -C /tmp/
    fi
  fi
  
  if [[ -d /tmp/hooks/prepare ]]; then
    case ${LANGUAGES} in
        'zh_CN')
                echo "执行安装前prepare定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Customize prepare scripts before installation execution..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "ལག་བསྟར་སྒྲིག་སྦྱོར་གོང་prepareངེས་སྲོལ་འཁྲབ་དེབ། ..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "執行安裝前prepare定制腳本..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠬᠤ ᠠᠴᠠ ᠡᠮᠦᠨᠡᠬᠢ prepare ᠦᠢᠯᠡᠳᠬᠦ ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "执行安装前prepare定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
    esac
    run_scripts_hooks /tmp/hooks/prepare || do_error hooks-prepare
    case ${LANGUAGES} in
        'zh_CN')
                echo "安装前prepare定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Prepare custom script execution before installation completed" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "སྒྲིག་སྦྱོར་གོང་prepareངེས་སྲོལ་འཁྲབ་དེབ་ལག་བསྟར་ལེགས་སྒྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "安裝前prepare定制腳本執行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠬᠤ ᠠᠴᠠ ᠡᠮᠦᠨᠡ prepare ᠦᠢᠯᠡᠳᠬᠦ ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠢ᠋ ᠬᠡᠷᠡᠭᠵᠢᠭᠦᠯᠵᠦ ᠳᠠᠭᠤᠰᠭᠠᠪᠠ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "安装前prepare定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
    esac
  fi
  
  run_scripts_dir prepare

}

# 运行 in_chroot 脚本
do_run_target() {
  mkdir -p /target/${IN_TARGET}
  cp utils /tmp/ky-installer.cfg /target/tmp
  cp -r k.sh in_chroot /target/${IN_TARGET}
  
  if [[ -d /tmp/hooks/in_chroot ]]; then
     cp -r /tmp/hooks/in_chroot /target/tmp
  fi

  chroot /target bash ${IN_TARGET}/k.sh >>${LOG_FILE} 2>&1
}

# 运行 after 脚本
do_after() {
  run_scripts_dir after

  if [[ -d /cdrom/hooks/after ]]; then
    case ${LANGUAGES} in
        'zh_CN')
                echo "执行安装后after定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Execute customized script after installation..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "ལག་བསྟར་སྒྲིག་སྦྱོར་རྗེས་afterངེས་སྲོལ་འཁྲབ་དེབ།..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "執行安裝後after定制腳本..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠬᠤ ᠶᠢ ᠬᠡᠷᠡᠭᠵᠢᠭᠦᠯᠦᠭᠰᠡᠨ ᠦ ᠳᠠᠷᠠᠭ᠎ᠠ after ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠦᠢᠯᠡᠳᠬᠦ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "执行安装后after定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
    esac
    run_scripts_hooks /cdrom/hooks/after || do_error hooks-after
    case ${LANGUAGES} in
        'zh_CN')
                echo "安装后after定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Custom script execution is completed after installation" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "སྒྲིག་སྦྱོར་རྗེས་afterངེས་སྲོལ་འཁྲབ་དེབ་ལག་བསྟར་ལེགས་སྒྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "安裝後after定制腳本執行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠭᠰᠠᠨ ᠤ ᠳᠠᠷᠠᠭᠠ after ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠦᠢᠯᠡᠳᠬᠦ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "安装后after定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
    esac
  fi

  if [[ -d /tmp/hooks/after ]]; then
    case ${LANGUAGES} in
        'zh_CN')
                echo "执行安装后after定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Execute customized script after installation..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "ལག་བསྟར་སྒྲིག་སྦྱོར་རྗེས་afterངེས་སྲོལ་འཁྲབ་དེབ།..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "執行安裝後after定制腳本..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠬᠤ ᠶᠢ ᠬᠡᠷᠡᠭᠵᠢᠭᠦᠯᠦᠭᠰᠡᠨ ᠦ ᠳᠠᠷᠠᠭ᠎ᠠ after ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠦᠢᠯᠡᠳᠬᠦ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "执行安装后after定制脚本..." >>${LOG_PROGRESS_FILE}
                ;;
    esac
    run_scripts_hooks /tmp/hooks/after || do_error hooks-after
    case ${LANGUAGES} in
        'zh_CN')
                echo "安装后after定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Custom script execution is completed after installation" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "སྒྲིག་སྦྱོར་རྗེས་afterངེས་སྲོལ་འཁྲབ་དེབ་ལག་བསྟར་ལེགས་སྒྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "安裝後after定制腳本執行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠰᠠᠭᠤᠯᠭᠠᠭᠰᠠᠨ ᠤ ᠳᠠᠷᠠᠭᠠ after ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠦᠢᠯᠡᠳᠬᠦ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "安装后after定制脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
    esac
  fi
}

# 执行 post 脚本
do_cdrom_post_actions() {
  
  case ${LANGUAGES} in
        'zh_CN')
                echo "执行postactions 脚本..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Execute postactions script..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "ལག་བསྟར་postactionsའཁྲབ་དེབ།..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "執行postactions腳本..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "postactions ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠢ᠋ ᠬᠡᠷᠡᠭᠵᠢᠭᠦᠯᠬᠦ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "执行postactions 脚本..." >>${LOG_PROGRESS_FILE}
                ;;
  esac

  if [[ -f /cdrom/.kylin-post-actions ]] && [[ "${is_ghost}" != "true" ]]; then
    cp /cdrom/.kylin-post-actions /target/${IN_TARGET}
    chroot /target bash ${IN_TARGET}/.kylin-post-actions >>${LOG_FILE} 2>&1 || do_error post-actions
  fi

  if [[ -f /cdrom/.kylin-post-actions-nochroot ]] && [[ "${is_ghost}" != "true" ]]; then
    bash /cdrom/.kylin-post-actions-nochroot >>${LOG_FILE} 2>&1 || do_error post-actions-nochroot
  fi
  
  case ${LANGUAGES} in
        'zh_CN')
                echo "postactions 脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Postactions script execution completed" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "Postactionsའཁྲབ་དེབ་ལག་བསྟར་ལེགས་སྒྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "postactions腳本執行完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "postactions ᠦᠯᠢᠭᠡᠷ ᠳᠡᠪᠲᠡᠷ ᠢ᠋ ᠬᠡᠷᠡᠭᠵᠢᠭᠦᠯᠵᠦ ᠳᠠᠭᠤᠰᠭᠠᠪᠠ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "postactions 脚本执行完成" >>${LOG_PROGRESS_FILE}
                ;;
  esac
}

# 拷贝日志
do_copy_log() {
  msg "正在拷贝安装日志..."
  cp -r /var/log/installer/ /target/var/log/
}

#离线包升级更新
do_update_installer() {
    kpb_list=$(find /cdrom/patches/ | grep .kpb ) || true
    if [[ -n ${kpb_list} ]]; then
        for kpb_one in $kpb_list
        do
                mkdir -p /target/patches
                cp -a ${kpb_one} /target/patches
                mkdir -p /target/kbp
                target_kpb="/target/patches/*.kpb"
                mount  ${target_kpb} /target/kbp
                if [ -e /target/kbp/update-install.py ]; then
                        echo "start exec update-install.py"
                        echo ${target_kpb}
                        chroot /target /bin/sh -c "python3 /kbp/update-install.py" || do_error kbp
                fi
        	umount /target/kbp
        	rm -rf /target/patches
        	rm -rf /target/kbp
        done
    else
        if grep -q "url=" /proc/cmdline; then
                kbp_patches=$(get_value patches)
                if [[ -n $kbp_patches ]]; then
                        cmd_cfg=$(cat /proc/cmdline)
                        url_ip=$(echo ${cmd_cfg##*url=}| awk '{print $1}')
                        url_ip=${url_ip%/*}
                        mkdir -p /target/patches
                        wget -P/target/patches ${url_ip}/${kbp_patches}
                        tar -xf /target/patches/${kbp_patches} -C /target/patches/
                        target_kpb=`find /target/patches | grep .kpb` || true
        		for kpb_two in $target_kpb
			do
				mkdir -p /target/kbp
                       		mount  ${kpb_two} /target/kbp
                        	if [ -e /target/kbp/update-install.py ]; then
                                	echo "start exec update-install.py"
                                	chroot /target /bin/sh -c "python3 /kbp/update-install.py" || do_error kbp
                        	fi
                        	umount /target/kbp
                        	rm -rf /target/patches
                        	rm -rf /target/kbp
			done
                fi
        fi
    fi
}


do_del_auti_mode() {
  if [[ -f /target/etc/xdg/autostart/oem-qc.desktop ]]; then
        rm -f /target/etc/xdg/autostart/oem-qc.desktop
  fi

  if [[ -f /target/usr/sbin/oem-second ]]; then
        rm -f /target/usr/sbin/oem-second
  fi
  
  if [[ -f /target/etc/xdg/autostart/oem-unlock.desktop ]]; then
        rm -f /target/etc/xdg/autostart/oem-unlock.desktop
  fi
  
  if [[ -f /target/usr/sbin/oem-unlock ]]; then
        rm -f /target/usr/sbin/oem-unlock
  fi
  if [[ -d /target/usr/share/oem-test ]]; then
       mv /target/usr/share/oem-test/ /usr/share/
  fi
  
  if [[ -e /usr/share/kylin-os-installer/99-ukui-installer.conf ]]; then
        mv /usr/share/kylin-os-installer/99-ukui-installer.conf /usr/share/lightdm/lightdm.conf.d
  fi

  if grep -q 'test-mode' /proc/cmdline; then
    oem_mode="oem-automatic"
  else
    oem_mode="oem"
  fi
 
  if [[ -f /target/usr/share/kylin-os-installer/96-kylin-os-installer.conf ]]; then
  	mv /target/usr/share/kylin-os-installer/96-kylin-os-installer.conf  /target/usr/share/lightdm/lightdm.conf.d
  else
  	cp -a /target/usr/share/applications/kylin-os-installer.desktop /target/etc/xdg/autostart || echo "kylin-os-installer.desktop is unexit" >>/target/var/log/installer/kylin-os-installer.log
  	sed -i "/Exec/d" /target/etc/xdg/autostart/kylin-os-installer.desktop
  	echo "Exec=/usr/bin/kylin-os-installer-${oem_mode}" >>/target/etc/xdg/autostart/kylin-os-installer.desktop
  fi



}

do_auti_mode() {
  
  if [[ -d /usr/share/oem-test ]]; then
       mv /usr/share/oem-test /target/usr/share/
  fi
  if [[ -f /target/usr/share/kylin-os-installer/data/oem-qc.desktop ]]; then
       cp -a /target/usr/share/kylin-os-installer/data/oem-qc.desktop /target/etc/xdg/autostart
       cp -a /target/usr/share/kylin-os-installer/data/oem-qc /target/usr/sbin/oem-qc

       if grep -q 'auti-only' /proc/cmdline; then
              cp -a /target/usr/share/kylin-os-installer/scripts/oem-second /target/usr/sbin/
              if [[ -f /cdrom/ky-installer.cfg ]]; then
                      cp /cdrom/ky-installer.cfg /target/usr/share/kylin-os-installer/ky-installer.cfg
              fi
       else
              cp -a /target/usr/share/kylin-os-installer/scripts/oem-second /target/usr/sbin/
       fi
   fi
   
   if [[ -f /target/usr/share/kylin-os-installer/data/oem-unlock.desktop ]]; then
              cp -a /target/usr/share/kylin-os-installer/data/oem-unlock.desktop /target/etc/xdg/autostart
              cp -a /target/usr/share/kylin-os-installer/data/oem-unlock /target/usr/sbin/oem-unlock
   fi

   if [[ -f /target/etc/xdg/autostart/kylin-os-installer.desktop ]]; then
        rm -f /target/etc/xdg/autostart/kylin-os-installer.desktop
   fi
   
   if [[ -f /target/usr/share/lightdm/lightdm.conf.d/96-kylin-os-installer.conf ]]; then
        mv /target/usr/share/lightdm/lightdm.conf.d/96-kylin-os-installer.conf /target/usr/share/kylin-os-installer
   fi


}

# 出厂备份
do_factory_backup() {
  
  case ${LANGUAGES} in
        'zh_CN')
                echo "正在进行系统出厂备份..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "System factory backup in progress..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "བཞིན་ལ་མ་ལག་ནས་ཐོན་པའི་备份། ..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "正在進行系統出廠備份..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠶᠠᠭ ᠰᠢᠰᠲ᠋ᠡᠮ ᠤᠨ ᠦᠢᠯᠡᠳᠪᠦᠷᠢ ᠡᠴᠡ ᠭᠠᠷᠭᠠᠬᠤ ᠨᠦᠬᠡᠴᠡᠯᠡᠯ ᠢ ᠶᠠᠪᠤᠭᠳᠠᠭᠤᠯᠵᠤ ᠪᠠᠶᠢᠨᠠ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "正在进行系统出厂备份..." >>${LOG_PROGRESS_FILE}
                ;;
  esac

  #删除出厂备份中的审核模式
  if grep -q 'auti-mode' /proc/cmdline; then
  	do_del_auti_mode  
  fi

  umount /target/backup
  msg "正在进行系统备份..."
  ## 工厂备份
  /usr/bin/backup-auto --factorybackup /target /backup || true
  msg "系统备份完成"

  #还原出厂备份中的审核模式
  if grep -q 'auti-mode' /proc/cmdline; then
        do_auti_mode
  fi
  
  case ${LANGUAGES} in
        'zh_CN')
                echo "系统出厂备份完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "System factory backup completed" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "མ་ལག་ནས་ཐོན་པའི་备份ལེགས་གྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "系統出廠備份完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo "ᠱᠢᠰᠲ᠋ᠧᠮ ᠦ᠋ᠨ ᠦᠢᠯᠡᠳᠪᠦᠷᠢ ᠡᠴᠡ ᠭᠠᠷᠭᠠᠬᠤ ᠪᠡᠯᠡᠳᠭᠡᠯ ᠠᠵᠢᠯ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "系统出厂备份完成" >>${LOG_PROGRESS_FILE}
                ;;
  esac

}

do_install_success() {
  if [[ "${is_990_9a0}" == "true" ]]; then
    mkdir -p /target/boot/efi
    touch /target/boot/efi/INSTALL_SUCCESS
  fi
}

handle_factory_backup() {
  factory_backup=$(get_value factory-backup)
  if [[ -f /target/etc/.bootinfo ]]; then
    if grep -q 'factory-backup' /proc/cmdline; then
      do_factory_backup
    elif [[ $factory_backup -eq 1 ]]; then
      do_factory_backup
    fi
  fi
}

do_finished() {
  echo 'finished'
  touch /tmp/z-finished-install
}

do_os_prober() {
  dmsetup ls |while read line
  do
        if [ "$line" = "No devices found" ];then
                break
        fi
        PART=`echo "$line"|awk '{print $1}'`
        dmsetup remove $PART || true
  done
}

do_select_disk() {
  if grep -q automatic /proc/cmdline; then
        devpath=$(get_value devpath)
        if [[ -z $devpath ]]; then
                 auto_disk >>${LOG_FILE}
        elif [[ $devpath == "min" ]]; then
       		 auto_disk_min >>${LOG_FILE}
        elif [[ $devpath == "speed" ]]; then
       		 auto_disk_speed >>${LOG_FILE}
	elif [[ $devpath == "M.2_0" ]]; then
                 auto_disk_slot $devpath >>${LOG_FILE}
        elif [[ $devpath == "M.2_1" ]]; then
                 auto_disk_slot $devpath >>${LOG_FILE} 
	else
                 get_disk  >>${LOG_FILE}
        fi
  elif grep -qE 'ple-mode|test-mode' /proc/cmdline; then
        devpath=$(get_value devpath)
        if [[ -z $devpath ]]; then
                 auto_disk >>${LOG_FILE}
        elif [[ $devpath == "min" ]]; then
       		 auto_disk_min >>${LOG_FILE}
        elif [[ $devpath == "speed" ]]; then
       		 auto_disk_speed >>${LOG_FILE} 
	elif [[ $devpath == "M.2_0" ]]; then
                 auto_disk_slot $devpath >>${LOG_FILE}
        elif [[ $devpath == "M.2_1" ]]; then
                 auto_disk_slot $devpath >>${LOG_FILE}
        else
                 get_disk  >>${LOG_FILE}
        fi
  fi

  data_device=$(get_value data-device)
  if [[ ${data_device} == "auto" ]]; then
	automatic=$(get_value automatic-installation)
	if [[ "$automatic" == "0" ]]; then
  		echo "GUI custom installation"
	else 
		get_data_disk >>${LOG_FILE}
	fi
  fi
}

do_apps_third() {
  if [[ -d /cdrom/apps-third ]] && [[ "${is_ghost}" != "true" ]]; then
  	if [[ -d /target/etc/apt/sources.list.d ]]; then
		echo "/target/etc/apt/sources.list.d is exit"
	else
		mkdir -p /target/etc/apt/sources.list.d
	fi
	echo "deb [trusted=yes] file:///opt/apps-third/ ./" >/target/etc/apt/sources.list.d/kylin-apps-third.list
	if [[ -f /target/etc/apt/sources.list ]]; then
                mv /target/etc/apt/sources.list /target/etc/apt/sources.list.bak
        fi
        chroot /target apt-get update --allow-insecure-repositories -y || true
        if [[ -f /target/etc/apt/sources.list.bak ]]; then
                mv /target/etc/apt/sources.list.bak /target/etc/apt/sources.list
        fi
  fi
}

# 解决os-prober问题
do_os_prober >>${LOG_FILE} 2>&1

#安装自动选择磁盘安装方案
do_select_disk >>${LOG_FILE} 2>&1

# 运行 prepare 阶段脚本
do_prepare >>${LOG_FILE} 2>&1

# 执行 target inchroot 脚本
do_run_target

# 运行 after 阶段脚本
do_after >>${LOG_FILE} 2>&1

# 执行 postactions 脚本
do_cdrom_post_actions

# 执行离线包升级
do_update_installer >>${LOG_FILE} 2>&1

#配置apps-third源
do_apps_third >>${LOG_FILE} 2>&1

# 拷贝安装日志
do_copy_log

do_install_success

# 处理出厂备份
handle_factory_backup >>${LOG_FILE} 2>&1

case ${LANGUAGES} in
        'zh_CN')
                echo "系统安装完成前的准备..." >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "Preparation before system installation is completed..." >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "མ་ལག་སྒྲིག་སྦྱོར་ལེགས་གྲུབ་སྔོན་གྱི་གྲ་སྒྲིག།..." >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "系統安裝完成前的準備..." >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo " ᠰᠢᠰᠲ᠋ᠡᠮ ᠤᠨ ᠰᠠᠭᠤᠯᠭᠠᠯᠲᠠ ᠳᠠᠭᠤᠰᠬᠤ ᠠᠴᠠ ᠡᠮᠦᠨᠡᠬᠢ ᠪᠡᠯᠡᠳᠬᠡᠯ..." >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "系统安装完成前的准备..." >>${LOG_PROGRESS_FILE}
                ;;
esac

#创建安装成功标准
touch /target/var/log/installer/INSTALL_SUCCESS

# 卸载文件系统
umount_all

do_finished

case ${LANGUAGES} in
        'zh_CN')
                echo "系统安装完成" >>${LOG_PROGRESS_FILE}
                ;;
        'en_US')
                echo "System installation completed" >>${LOG_PROGRESS_FILE}
                ;;
        'bo_CN')
                echo "མ་ལག་སྒྲིག་སྦྱོར་ལེགས་གྲུབ།" >>${LOG_PROGRESS_FILE}
                ;;
        'zh_HK')
                echo "系統安裝完成" >>${LOG_PROGRESS_FILE}
                ;;
        'mn_MN')
                echo " ᠰᠢᠰᠲ᠋ᠡᠮ ᠤᠨ ᠰᠠᠭᠤᠯᠭᠠᠯᠲᠠ ᠪᠡᠶᠡᠯᠡᠭᠳᠡᠬᠦ" >>${LOG_PROGRESS_FILE}
                ;;
              *)
                echo "系统安装完成" >>${LOG_PROGRESS_FILE}
                ;;
esac

